############################################################################
##
## Copyright (c) 2000 BalaBit IT Ltd, Budapest, Hungary
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
##
## $Id: Listener.py,v 1.32 2001/01/02 11:34:42 bazsi Exp $
##
## Author  : Bazsi
## Auditor : kisza
## Last version: 1.15
## Notes:
##   - comments in English
##
############################################################################

from Zorp import *
from Session import MasterSession
from Service import services
from traceback import print_exc

listen_hook = None
unlisten_hook = None

class Listener:
	"""Class to listen on the selected address.
	
	This is the starting point of Zorp services. It listens on the
	given port, and when a connection is accepted it starts a session
	and the given service.
                   
	Attributes:
	
	  listen --       A Zorp.Listen instance
	  
	  service --      the service to be started
	  
	  bindto --       bind address
	  
	  local --        local address where the listener is bound
	"""
	listen = None  
	service = None

	def __init__(self, bindto, service):
		"""Constructor to initialize a Listen instance
		
		Creates the instance, sets the initial attributes, and
		starts the listener

		Arguments:

		  self -- this instance
		  
		  bindto --  the address to bind to
		  
		  service -- the service name to start
		"""
		global listen_hook
		
		try:
			self.service = services[service]
		except KeyError:
			raise ServiceException, "Service %s not found" % (service,)
		self.bindto = bindto
	        self.listen = Listen(bindto, self.accepted, 0)
	        self.local = self.listen.local
	        if listen_hook:
		        listen_hook(self)
	        self.listen.start()
	        
	def __del__(self):
		global unlisten_hook
		
		if unlisten_hook:
			unlisten_hook(self)

	def accepted(self, client, fd):
		"""Callback to inform the python layer about incoming connections.
		
		This callback is called by the core when a connection is 
		accepted. Creates and starts a service.

		Arguments:

		  self   --  this instance
		  
		  client --  the address of the client
		  
		  fd     --  the fd of the connection to the client
		  
		Returns:
		
		  TRUE if the connection is accepted
		  
		Notes:
		
		  This function does the access control checks (in
		  setService and setClient), sets initial session
		  parameters, logs a debug message, and starts the
		  service.  Exceptions raised due to policy violations
		  are handled here. (client not allowed to use this
		  service, or couldn't authenticate himself)
		  
		"""
		
		session = None
		try:
			session = MasterSession()
			
			session.setService(self.service)
			session.setClient(fd, client)
			session.client_local = self.local

			#service = self.getService(session)
			
			log("core.debug", 8, "accepted a connection from python, client=%s" % client.format())
			session.service.startInstance(session)
			return Z_ACCEPT
		except ZoneException, s:
			log("core.policy", 1, "%s: Zone not found: %s" % (session.session_id, s))
		except DACException:
			log("core.policy", 1, "%s: DAC policy violation" % (session.session_id))
		except MACException:
			log("core.policy", 1, "%s: MAC policy violation" % (session.session_id))
		except AuthException:
			log("core.policy", 1, "%s: Authentication failure" % (session.session_id))
		except:
			print_exc()
			
		if session != None: 
			session.destroy()

		return Z_REJECT

	def getService(self, session):
		"""Returns the service associated with the listener

		Returns the service to start.

		Arguments:

		  self    -- this instance
		  
		  session -- session reference

		Notes:

		  FIXME: this function is not yet used
		"""
		return self.service

	def destroy(self):
		"""Stops the listener on the given port

		Calls the destroy method of the low-level object

		Arguments:
		
		  self -- this instance
		  
		"""
		self.listen.destroy()

#
#
#class MultiListen(Listen):
#	"""
#	"""
#	services = None
#	
#	def __init__(self, bindto, services, default_service):
#		Listen.__init__(self, bindto, default_service)
#		self.services = services
#	
#	def getService(self, session):
#		try:
#			return self.services[session.client_zone];
#		except KeyError:
#			return self.service
#
