############################################################################
##
## Copyright (c) 2000 BalaBit IT Ltd, Budapest, Hungary
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
##
## $Id: Proxy.py,v 1.27 2000/12/28 17:45:07 bazsi Exp $
##
## Author  : Bazsi
## Auditor : kisza
## Last version: 1.13
## Notes:
##   - comments in English
##
############################################################################

from Zorp import *
from traceback import print_exc
from Stream import Stream
import new

def proxyLog(self, type, level, msg):
	""" Log a proxy message.

	Log messages about proxies. Automatically put session_id into
	each message.

	Arguments:

	  self  -- should be a Proxy derivative

	  type  -- log class

	  level -- message verbosity level

	  msg   -- msg to log
	  
	"""
	#FIXME: performance check. (Check level before parsing string)
	try:
		mymsg = "%s: %s" % (self.session.session_id, msg)
	except:
		mymsg = "Error parsing log"
	log(type, level, mymsg)

class Proxy(ZorpProxy):
	"""Base class for the proxies.

	It is inherited from an ExtensionClass implemented in C. The
	exact proxy module is determined by the name parameter passed
	to the constructor.
	
	Attributes:
	
	  session      -- descriptor of the connection
	"""
	
	def __init__(self, name, session):
		"""Initializes a Proxy instance.

		Initializes a Proxy instance and sets attributes.
		
		Arguments:
		
		  name	  -- The type of the proxy
		  
		  session -- reference to the session
		"""
		self.session = session
		session.setProxy(name)
		log("core.session", 1, "%s: session_start, module=%s" % (session.session_id, name));
		try:
			ZorpProxy.__init__(self, name, session.session_id, session.client_stream)
		except:
			print_exc()
			session.destroy()


	def addPolicy(self, klass):
		"""Adds a policy to the proxy.

		This function extends this class with another parent
		class at runtime. It can be used to add methods
		to this event handler class based on some
		runtime variable.
		
		Arguments:

		  self        -- this instance
		  
		  klass	      -- the class implementing the policy

		"""
		self.__class__ = new.classobj(self.__class__.__name__, self.__class__.__bases__ + (klass,), self.__class__.__dict__)

	def connectServer(self, host, port):
		"""Callback method called when a connection established

		If the chainer is defined, it is called with the
		given host and port parameters to connect to the
		remote server.

		If there is no chainer defined, it tries to return
		the server-side fd from the session. If there's no
		server_fd it raises an error.

		Arguments:

		  self          -- this instance
		  
		  host		-- The address of the server
		  
		  port	       	-- The port of the server
		  
		Returns:
		
		  The descriptor of the server stream

		"""
		self.server_stream = None
		self.server_fd = -1
		if self.session.chainer == None:
			
			# we have no chainer, the server side fd
			# should be available by now, used in stacked
			# proxies
			
			if self.session.server_fd == -1 or self.session.server_stream == None:
				raise InternalError, "No chainer and server_fd is -1 or server_stream is None"				
		else:
			fd = self.session.chainer.connectServer(self.session, host, port)
			if fd == -1:
				self.session.server_stream = None
			else:
				self.session.server_stream = Stream(self.session.server_fd, self.session.owner.session_id + ": server")

		return self.session.server_stream

class DatagramProxy(ZorpProxy):
	"""Base class for datagram based protocols.

	This class is similar to [Proxy] with the exception that
	it's used for datagram based protocols.

	.. [Proxy] Proxy
	
	Attributes:

	  self          -- this instance
	  
	  session	-- the session we belong to
	"""
	session = None
	
	def __init__(self, name, session):
		"""Initializes a DatagramProxy instance.

		Sets attributes based on arguments.

		Arguments:

		  self          -- this instance
		  
		  name		-- The type of the proxy

		  session	-- The descriptor of the connection
		  
		"""
		self.session = session
		log("core.session", 1, "%s: session_start, module=%s" % (session.session_id, name));
		try:
			ZorpProxy.__init__(self, name, session.session_id, session.client_stream)
		except:
			session.destroy()
