############################################################################
##
## Copyright (c) 2000 BalaBit IT Ltd, Budapest, Hungary
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
##
## $Id: Zone.py,v 1.31 2000/12/28 17:45:07 bazsi Exp $
##
## Author  : Bazsi
## Auditor : kisza
## Last version: 1.11
## Notes:
##   - comments in English
##
############################################################################

from Zorp import *
from Domain import InetDomain
from traceback import print_exc
import types

#labelset_class = LabelSet;
zones = {}
root_zone = None

class RootZone:
	"""
	FIXME: abstract access control out of Zone
	"""
	def __init__(self, name, inbound_services=None, outbound_services=None, admin_parent=None, umbrella=0):
		global zones
		
		if zones.has_key(name):
			raise ZoneException, 'Zone %s duplicate name' % name
		zones[name] = self
		self.name = name
		self.addr_parent = None
		self.addr_children = []
		self.umbrella = umbrella
		self.admin_children = []
		if admin_parent:
			self.admin_parent = zones[admin_parent]
			self.admin_parent.addAdminChild(self)
		else:
			self.admin_parent = None

                self.inbound_services = {}
                self.outbound_services = {}
                if outbound_services != None:
                        for i in outbound_services:
                                self.outbound_services[i] = 1
				debug(5, "Zone(%s): outbound service=%s" % (self.name, i))
        
                if inbound_services != None:            
                        for i in inbound_services:
                                self.inbound_services[i] = 1
				debug(5, "Zone(%s): inbound service=%s" % (self.name, i))
		
	def findDomain(self, domain):
		for x in self.addr_children:
			if x.domain is domain:
				return x
			
		# we never return self, because RootZone is not a real zone
		raise ZoneException, "Zone not found for domain %s" % (domain, )

	def findZone(self, address):
		for x in self.addr_children:
			try:
				if address < x.address:
					return x.findZone(address)
			except ValueError:
				pass
		raise ZoneException, "Address Domain not found for address %s" % (address, )
		
	def addAdminChild(self, child):
		child.admin_parent = self
		self.admin_children.append(child)

	def setAdminParent(self, parent):
		self.admin_parent = parent
		
	def iterAdminChildren(self, fn, parm = None):
		for x in self.admin_children[:]:
			fn(parm, self, x)

	def addAddrChild(self, child):
		child.addr_parent = self
		self.addr_children.append(child)

	def delAddrChild(self, child):
		child.addr_parent = None
		self.addr_children.remove(child)

	def iterAddrChildren(self, fn, parm = None):
		for x in self.addr_children[:]:
			fn(parm, self, x)

	def __str__(self):
		return "RootZone(%s)" % self.name

        def isInboundServicePermitted(self, session):
                """
                """
                try:
                        t = self.inbound_services["*"]
                except KeyError:
                        t = 0
                
                try:
                        t = self.inbound_services[session.service.name]
                except KeyError:
                        pass
                
                if t:
                        return Z_ACCEPT
                else:           
                	if self.admin_parent:
                		return self.admin_parent.isInboundServicePermitted(session)
                        return Z_REJECT
                
        def isOutboundServicePermitted(self, session):
                """
                """
                try:
                        t = self.outbound_services["*"]
                except KeyError:
                        t = 0
                
                try:
                        t = self.outbound_services[session.service.name]
                except KeyError:
                        pass
                
                if t:
                        return Z_ACCEPT
                else:           
                	if self.admin_parent:
                		return self.admin_parent.isOutboundServicePermitted(session)
                        return Z_REJECT
                                                
class Zone(RootZone):
	def __init__(self, name, addr, inbound_services = None, outbound_services = None, admin_parent = None, umbrella = 0, domain = None):
		"""
		"""
		
		RootZone.__init__(self, name, inbound_services, outbound_services, admin_parent, umbrella)
		
		if domain is None:
			raise ValueError, "domain must be defined"
		
		self.domain = domain
		if type(addr) == types.ListType:
			i = 0
			while i < len(addr):
                        	Zone("%s-#%u" % (name, i), addr[i], admin_parent=self.name, domain=domain)
                        	i = i + 1
			addr = None
		
		if addr:
			self.address = domain(addr)
			self.setAddrRelatives()
		else:
			self.address = None

	def setAddrRelatives(self):

		def moveChildren(self, parent, zone):
			try:
				if zone.address < self.address:
					parent.delAddrChild(zone)
					self.addAddrChild(zone)
			except ValueError:
				pass


		try:
			root_domain = root_zone.findDomain(self.domain)
		except ZoneException:
			root_zone.addAddrChild(self)
			return

		try:
			zone = None
			zone = root_domain.findZone(self.address)
		except ZoneException:
			pass

		if zone == None:
			# current root domain is smaller than self
			root_zone.delAddrChild(root_domain)
			root_zone.addAddrChild(self)
			self.addAddrChild(root_domain)
			return
		if self.address == zone.address:
			raise NotImplementedError

				
		zone.iterAddrChildren(moveChildren, self)
		zone.addAddrChild(self)
				
	def findZone(self, addr):
		for zone in self.addr_children:
			try:
				return zone.findZone(addr)
			except ZoneException:
				pass
		try:
			if addr <= self.address:
				return self
		except ValueError:
			pass
		raise ZoneException, "Zone not found for address %s" % (addr, )

	def __str__(self):
		return "Zone(%s, %s)" % (self.name, self.address)
		

class InetZone(Zone):
	def __init__(self, name, addr, inbound_services = None, outbound_services = None, admin_parent = None, umbrella = 0):
		Zone.__init__(self, name, addr, inbound_services, outbound_services, admin_parent, umbrella, InetDomain)


root_zone = RootZone("root")

